#!/usr/bin/env node
"use strict";
/* eslint-disable no-console */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Logger_1 = require("../Logger");
const jiti_1 = __importDefault(require("jiti"));
const node_fs_1 = require("node:fs");
const node_path_1 = __importDefault(require("node:path"));
const helpers_1 = require("yargs/helpers");
const yargs_1 = __importDefault(require("yargs/yargs"));
const log = Logger_1.Logger.child({
    namespace: 'turbowatch',
});
const main = async () => {
    const argv = (0, yargs_1.default)((0, helpers_1.hideBin)(process.argv))
        .command('$0 [turbowatch.ts]', 'Start Turbowatch', (commandYargs) => {
        commandYargs.positional('turbowatch.ts', {
            alias: 'source',
            default: 'turbowatch.ts',
            describe: 'Script with Turbowatch instructions.',
            type: 'string',
        });
    })
        .parseSync();
    let resolvedPath;
    const providedPath = node_path_1.default.resolve(process.cwd(), argv.source);
    const possiblePaths = [providedPath];
    if (node_path_1.default.extname(providedPath) === '') {
        possiblePaths.push(providedPath + '.ts', providedPath + '.js');
    }
    for (const possiblePath of possiblePaths) {
        if ((0, node_fs_1.existsSync)(possiblePath)) {
            resolvedPath = possiblePath;
        }
    }
    if (!resolvedPath) {
        console.error('%s not found', providedPath);
        process.exitCode = 1;
        return;
    }
    const userScript = (0, jiti_1.default)(__filename)(resolvedPath)
        .default;
    if (typeof (userScript === null || userScript === void 0 ? void 0 : userScript.then) !== 'function') {
        console.error('Expected user script to export an instance of TurbowatchController');
        process.exitCode = 1;
        return;
    }
    const turbowatchController = await userScript;
    process.once('SIGINT', () => {
        log.warn('received SIGINT; gracefully terminating');
        void turbowatchController.shutdown();
    });
};
void main();
//# sourceMappingURL=turbowatch.js.map