"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.watch = void 0;
const generateShortId_1 = require("./generateShortId");
const Logger_1 = require("./Logger");
const subscribe_1 = require("./subscribe");
const testExpression_1 = require("./testExpression");
const chokidar = __importStar(require("chokidar"));
const serialize_error_1 = require("serialize-error");
const throttle_debounce_1 = require("throttle-debounce");
const log = Logger_1.Logger.child({
    namespace: 'watch',
});
const watch = (configurationInput) => {
    var _a, _b, _c, _d;
    const { project, triggers, debounce: userDebounce, } = {
        // as far as I can tell, this is a bug in unicorn/no-unused-properties
        // https://github.com/sindresorhus/eslint-plugin-unicorn/issues/2051
        // eslint-disable-next-line unicorn/no-unused-properties
        debounce: {
            wait: 1000,
        },
        ...configurationInput,
    };
    const abortController = new AbortController();
    const abortSignal = abortController.signal;
    let discoveredFileCount = 0;
    const indexingIntervalId = setInterval(() => {
        log.trace('indexed %d %s...', discoveredFileCount, discoveredFileCount === 1 ? 'file' : 'files');
    }, 5000);
    const subscriptions = [];
    const watcher = chokidar.watch(project);
    const shutdown = async () => {
        clearInterval(indexingIntervalId);
        abortController.abort();
        for (const subscription of subscriptions) {
            const { activeTask } = subscription;
            if (activeTask === null || activeTask === void 0 ? void 0 : activeTask.promise) {
                await (activeTask === null || activeTask === void 0 ? void 0 : activeTask.promise);
            }
        }
        // eslint-disable-next-line promise/prefer-await-to-then
        await watcher.close();
    };
    if (abortSignal) {
        abortSignal.addEventListener('abort', () => {
            shutdown();
        }, {
            once: true,
        });
    }
    for (const trigger of triggers) {
        subscriptions.push((0, subscribe_1.subscribe)({
            abortSignal,
            expression: trigger.expression,
            id: (0, generateShortId_1.generateShortId)(),
            initialRun: (_a = trigger.initialRun) !== null && _a !== void 0 ? _a : true,
            interruptible: (_b = trigger.interruptible) !== null && _b !== void 0 ? _b : true,
            name: trigger.name,
            onChange: trigger.onChange,
            onTeardown: trigger.onTeardown,
            retry: (_c = trigger.retry) !== null && _c !== void 0 ? _c : {
                retries: 0,
            },
            throttleOutput: (_d = trigger.throttleOutput) !== null && _d !== void 0 ? _d : { delay: 1000 },
        }));
    }
    let queuedChokidarEvents = [];
    const evaluateSubscribers = (0, throttle_debounce_1.debounce)(userDebounce.wait, () => {
        const currentChokidarEvents = queuedChokidarEvents;
        queuedChokidarEvents = [];
        for (const subscription of subscriptions) {
            const relevantEvents = currentChokidarEvents.filter((chokidarEvent) => {
                return (0, testExpression_1.testExpression)(subscription.expression, chokidarEvent.path);
            });
            if (relevantEvents.length) {
                void subscription.trigger(relevantEvents);
            }
        }
    }, {
        noLeading: true,
    });
    let ready = false;
    const discoveredFiles = [];
    watcher.on('all', (event, path) => {
        if (ready) {
            queuedChokidarEvents.push({
                event,
                path,
            });
            evaluateSubscribers();
        }
        else {
            if (discoveredFiles.length < 10) {
                discoveredFiles.push(path);
            }
            discoveredFileCount++;
        }
    });
    return new Promise((resolve, reject) => {
        watcher.on('error', (error) => {
            log.error({
                error: (0, serialize_error_1.serializeError)(error),
            }, 'could not watch project');
            if (ready) {
                shutdown();
            }
            else {
                reject(error);
            }
        });
        watcher.on('ready', () => {
            ready = true;
            clearInterval(indexingIntervalId);
            if (discoveredFiles.length > 10) {
                log.trace({
                    files: discoveredFiles.slice(0, 10).map((file) => {
                        return file;
                    }),
                }, 'discovered %d files in %s; showing first 10', discoveredFileCount, project);
            }
            else {
                log.trace({
                    files: discoveredFiles.map((file) => {
                        return file;
                    }),
                }, 'discovered %d %s in %s', discoveredFileCount, discoveredFiles.length === 1 ? 'file' : 'files', project);
            }
            log.info('Initial scan complete. Ready for changes');
            for (const subscription of subscriptions) {
                if (subscription.initialRun) {
                    void subscription.trigger([]);
                }
            }
            resolve({
                shutdown,
            });
        });
    });
};
exports.watch = watch;
//# sourceMappingURL=watch.js.map